#ifndef ENVIRONMENT_H
#define ENVIRONMENT_H


#include <fstream.h>
#include <vector.h>

#include "element.h"


/** This is the base-class for all environments.
    It provides a common interface for reading an environment from an
    input filestream and writing it to an output filestream.
    There is also a method, which returns the number of elements in the
    environment and a method which return-value indicates, if the elements
    of the environment should be written one-by-one or not.
*/
class Environment {
  public:
    /// Default constructor
    Environment();

    /// Destructor
    virtual ~Environment();


    /** This method builds the environment from the given input filestream.
	The input filestream will be read until an end-of-environment tag is
	reached.
	@param input The input filestream
    */
    virtual void read(ifstream *input);

    /** This method writes the environment to the given output filestream.
	@param output The output filestream
	@param parameter An integer-parameter, which has a special meaning for
	       each derivated environment.
    */
    virtual void write(ofstream *output,int parameter=0);

    /** This method indicates, if the elements of the environment should be
	written one-by-one.
	@return 1, if the elements should be written one-by-one, else 0.
    */
    virtual int shouldBeWrittenOneByOne();

    /** This method returns the number of elements in the environment.
	@return The number of elements
    */
    virtual int getNumberOfElements();


  protected:
    /// A vector which contains pointers to the elements.
    vector<Element*> elements_;
};

#endif //ENVIRONMENT_H
