#include "main.h"
#include "gridenvironment.h"
#include "imageelement.h"


GridEnvironment::GridEnvironment(int write_one_by_one) : Environment()
{
    columns_ = 0;
    rows_ = 0;
    write_one_by_one_ = write_one_by_one;
}

GridEnvironment::~GridEnvironment()
{
}


void GridEnvironment::read(ifstream *input)
{
    unsigned int line_of_environment = 1;

    input->getline(Buffer_,BUFFER_SIZE);
    Current_Line_Number_++;

    while (!strstr(Buffer_,"</GRID>")) {
        if (Buffer_[0] != '#') {
	    if (line_of_environment == 1) {
	        istrstream dimension(Buffer_);
		dimension >> columns_ >> rows_;

	    } else if (input->eof() || (strlen(Buffer_) == 0)) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There should be a </GRID> tag." << endl;
		Number_Of_Errors_++;
		return;

       	    } else {
	        elements_.push_back(new ImageElement(Buffer_));
	    }

	    line_of_environment++;
	}

	input->getline(Buffer_,BUFFER_SIZE);
	Current_Line_Number_++;
    }

    if (line_of_environment == 1) {
        cerr << "Line " << Current_Line_Number_
	     << ": There should be at least one image." << endl;
	Number_Of_Errors_++;
    }
    if (line_of_environment-2 > columns_ * rows_) {
        cerr << "Line " << Current_Line_Number_
	     << ": There should be at most " << columns_ * rows_ << " images."
	     << endl;
	Number_Of_Errors_++;
    }
}


void GridEnvironment::write(ofstream *output,int number_of_images)
{
    if ((number_of_images == -1) ||
	(number_of_images > (signed)elements_.size())) {
        number_of_images = elements_.size();
    }

    *output << "<TABLE align=\"center\" width=\"90%\" cellpadding=\"10\" "
	    << "border=\"0\">" << endl;

    for (unsigned int i=0;i<rows_;i++) {
        *output << "<TR>" << endl;
        for (unsigned int j=0;j<columns_;j++) {
	    *output << "<TD width=\"" << 100/columns_ << "%\">";
	    if (i*columns_ + j < (unsigned)number_of_images) {
	        elements_[i*columns_ + j]->write(output);
	    }
	    *output << "</TD>" << endl;
	}
	*output << "</TR>" << endl;
    }

    *output << "</TABLE>" << endl;
}


int GridEnvironment::shouldBeWrittenOneByOne()
{
    return write_one_by_one_;
}
