#ifndef GRIDENVIRONMENT_H
#define GRIDENVIRONMENT_H


#include "environment.h"


/** This class represents a <GRID>-environment.
    Currently, the grid can only contain images (ImageElements) stored in an
    Element-vector (derived from Environment).
    Note : I'm not sure, if the grid should be able to hold any
    sub-environments like the class HBoxEnvironment. If not, the name of this
    class should be changed to "ImageGridEnvironment".
*/
class GridEnvironment : public Environment {
  public:
    /** Generates a new <GRID>-environment with the given parameters.
	@param write_one_by_one Should the images be written one-by-one ?
    */
    GridEnvironment(int write_one_by_one);

    /// Destructor
    ~GridEnvironment();


    /** This method generates a <GRID>-environment out of the given input
	filestream.
	The first line after "<GRID>" contains the number of columns and rows
	of the grid.
	The input filestream will be read until a </GRID>-tag is reached.
	@param input The input filestream, from where the images are extracted
    */
    void read(ifstream *input);

    /** This method writes the given number of images to the given output
	filestream.
	@param output The output filestream
	@param number_of_images The number of images that will be written.
	       If -1, all images will be written.
    */
    void write(ofstream *output,int number_of_images);

    /** This method indicates, if the images should be written one-by-one.
	@return 1, if the images should be written one-by-one, else 0.
    */
    int shouldBeWrittenOneByOne();


  private:
    /// The number of columns the grid will have.
    unsigned int columns_;

    /// The number of rows the grid will have.
    unsigned int rows_;

    /// 1, if the images should be written one-by-one, else 0.
    unsigned int write_one_by_one_;
};

#endif //GRIDENVIRONMENT_H
