#include <stdlib.h>
#include <string.h>

#include "main.h"
#include "hboxenvironment.h"
#include "itemizeenvironment.h"
#include "imageenvironment.h"
#include "rawenvironment.h"


HBoxEnvironment::HBoxEnvironment() : Environment()
{
    environments_.clear();
    number_of_elements_ = -1;
    write_one_by_one_ = 0;
}

HBoxEnvironment::~HBoxEnvironment()
{
    environments_.clear();
}


void HBoxEnvironment::read(ifstream *input)
{
    Environment *env;
    unsigned int line_of_environment = 1;

    input->getline(Buffer_,BUFFER_SIZE);
    Current_Line_Number_++;

    while (!strstr(Buffer_,"</HBOX>")) {
        if (Buffer_[0] != '#') {
	    if (strstr(Buffer_,"<ITEMIZE>")) {
	        env = new ItemizeEnvironment(0,0,0);
		env->read(input);

	    } else if (strstr(Buffer_,"<ITEMIZE STEP>")) {
	        env = new ItemizeEnvironment(1,0,0);
		env->read(input);
		if (env->getNumberOfElements() > number_of_elements_) {
		    number_of_elements_ = env->getNumberOfElements();
		}
		write_one_by_one_ = 1;

	    } else if (strstr(Buffer_,"</ITEMIZE>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's an <ITEMIZE> tag missing." << endl;
		Number_Of_Errors_++;
		
	    } else if (strstr(Buffer_,"<IMG>")) {
	        env = new ImageEnvironment();
		env->read(input);
		if (env->shouldBeWrittenOneByOne()) {
		    if (env->getNumberOfElements() > number_of_elements_) {
		        number_of_elements_ = env->getNumberOfElements();
		    }
		    write_one_by_one_ = 1;
		}

	    } else if (strstr(Buffer_,"</IMG>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's an <IMG> tag missing." << endl;
		Number_Of_Errors_++;

	    } else if (strstr(Buffer_,"<RAW>")) {
	        env = new RawEnvironment();
		env->read(input);

	    } else if (strstr(Buffer_,"</RAW>")) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There's an <RAW> tag missing." << endl;
		Number_Of_Errors_++;

	    } else if (input->eof() || (strlen(Buffer_) == 0)) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There should be a </HBOX> tag." << endl;
		Number_Of_Errors_++;
		return;

       	    } else {
	        cerr << "Line " << Current_Line_Number_
		     << ": Unsupported environment inside the <HBOX>." << endl;
		Number_Of_Errors_++;
	    }

	    line_of_environment++;
	}
	environments_.push_back(env);
	input->getline(Buffer_,BUFFER_SIZE);
	Current_Line_Number_++;
    }

    if (line_of_environment == 1) {
        cerr << "Line " << Current_Line_Number_
	     << ": There should be at least one sub-environment." << endl;
	Number_Of_Errors_++;
    }
}


void HBoxEnvironment::write(ofstream *output,int parameter)
{
    *output << "<TABLE align=\"center\" width=\"90%\" cellpadding=\"20\" "
	    << "border=\"0\"><TR>" << endl;

    for (unsigned int i=0;i<environments_.size();i++) {
        *output << "<TD width=\"50%\" valign=\"top\">" << endl;
	environments_[i]->write(output,
				(environments_[i]->shouldBeWrittenOneByOne() ?
				 parameter : -1));
	*output << "</TD>" << endl;
    }

    *output << "</TR></TABLE>" << endl;
}


int HBoxEnvironment::shouldBeWrittenOneByOne()
{
    return write_one_by_one_;
}

int HBoxEnvironment::getNumberOfElements()
{
    return number_of_elements_;
}
