#ifndef HBOXENVIRONMENT_H
#define HBOXENVIRONMENT_H


#include "environment.h"


/** This class represents a <HBOX>-environment.
    This environment contains a vector which can hold any other environments.
    These sub-environments will be written side by side on the slide.
    Note : The member-variable elements_ (derivated from Environment) is not
    used in this class.
*/
class HBoxEnvironment : public Environment {
  public:
    /// Default constructor
    HBoxEnvironment();

    /// Destructor
    ~HBoxEnvironment();


    /** This method generates a <HBOX>-environment out of the given input
	filestream.
	The sub-environments will be read from the input filestream until a
	</HBOX>-tag is reached.
	@param input The input filestream, from where the sub-environments are
	       extracted
    */
    void read(ifstream *input);

    /** This method writes all sub-environments to the given output filestream.
	The integer-parameter will be passed to all sub-environment, that
	should be written one-by-one.
	@param output The output filestream
	@param parameter The parameter that will be passed
    */
    void write(ofstream *output,int parameter);

    /** This method indicates, if at least one sub-environment should be
	written one-by-one.
	@return 1, if at least one sub-environment should be written
	        one-by-one, else 0.
    */
    int shouldBeWrittenOneByOne();

    /** This method returns the maximum of the number of elements of the
	embedded sub-environments that should be written one-by-one.
	@return The maximum number of elements
    */
    int getNumberOfElements();


  private:
    /// A vector that holds pointers to the sub-environments.
    vector<Environment*> environments_;

    /** The maximum of the number of elements of all embedded sub-environments
	that should be written one-by-one.
	If no such sub-environment exist, the value is -1.
    */
    int number_of_elements_;

    /** If at least one embedded sub-environment should be written one-by-one,
	the value is 1.	If no such sub-environment exist, the value is 0.
     */
    int write_one_by_one_;
};

#endif //HBOXENVIRONMENT_H
