#ifndef IMAGEENVIRONMENT_H
#define IMAGEENVIRONMENT_H


#include "environment.h"


/** This class represents an <IMG>-environment.
    This environment contains at least one image (ImageElement) stored in an
    Element-vector (derived from Environment).
*/
class ImageEnvironment : public Environment {
  public:
    /// Default constructor
    ImageEnvironment();

    /// Destructor
    ~ImageEnvironment();


    /** This method generates an <IMG>-environment out of the given input
	filestream.
	The input filestream will be read until an </IMG>-Tag is reached.
	@param input The input filestream, from where the images are extracted
    */
    void read(ifstream *input);

    /** This method writes the given image to the given output filestream.
	If the parameter which_image is -1, the first image will be written.
	If which_image is bigger than the total number of images in the vector,
	the last image will be written.
	@param output The output filestream
	@param which_image Which image from the list should be written.
    */
    void write(ofstream *output,int which_image);

    /** This method indicates, if the images should be written one-by-one,
	or if only the first image should be written.
	@return 1, if the images should be written one-by-one, else 0.
    */
    int shouldBeWrittenOneByOne();
};

#endif //IMAGEENVIRONMENT_H
