#ifndef ITEMIZEELEMENT_H
#define ITEMIZEELEMENT_H


#include "element.h"
#include "itemizeenvironment.h"


/** This class represents one item of an <ITEMIZE>-environment.
    It contains a line of text (derivated from Element) and may contain an
    embedded <ITEMIZE>-subenvironment.
*/
class ItemizeElement : public Element {
  public:
    /// How the item should be printed : NORMAL or WEAK
    enum Style { NORMAL, WEAK };


    /** Creates a new item.
	@param text The text of the item
    */
    ItemizeElement(const char *text);

    /// Destructor
    ~ItemizeElement();


    /** This method returns the number of characters of the item.
	The return-value is used to compute the left margin of the HTML-output.
	@return The number of characters of the item
    */
    unsigned int getLength();

    /** This method writes the item to the given output filestream.
	@param output The output filestream
	@param style How the item should be printed (NORMAL or WEAK)
    */
    void write(ofstream *output,int style);


  public:
    /// A pointer to an <ITEMIZE>-subenvironment
    ItemizeEnvironment *sub_itemize_;
};

#endif //ITEMIZEELEMENT_H
