#include "main.h"
#include "itemizeenvironment.h"
#include "itemizeelement.h"


ItemizeEnvironment::ItemizeEnvironment(int write_one_by_one,unsigned int level,
				       int with_left_margin) : Environment()
{
    write_one_by_one_ = write_one_by_one;
    level_ = level;
    with_left_margin_ = with_left_margin;
    number_of_items_ = 0;
}

ItemizeEnvironment::~ItemizeEnvironment()
{
}


void ItemizeEnvironment::read(ifstream *input)
{
    unsigned int line_of_environment = 1;
    ItemizeElement *item = NULL;

    input->getline(Buffer_,BUFFER_SIZE);
    Current_Line_Number_++;

    while (!strstr(Buffer_,"</ITEMIZE>")) {
        if (Buffer_[0] != '#') {
	    if (strstr(Buffer_,"<ITEMIZE>")) {
	        if (item) {
		    item->sub_itemize_ = new ItemizeEnvironment(0,level_+1);
		    item->sub_itemize_->read(input);
		    number_of_items_++;
		} else {
		    cerr << "Line " << Current_Line_Number_
			 << ": There must be at least one item." << endl;
		    Number_Of_Errors_++;
		    read(input);  // Follow the sub-itemization recursively
		}
		
	    } else if (strstr(Buffer_,"<ITEMIZE STEP>")) {
	        if (item) {
		    item->sub_itemize_ = new ItemizeEnvironment(1,level_+1);
		    item->sub_itemize_->read(input);
		    number_of_items_ += item->sub_itemize_->getNumberOfElements();
		} else {
		    cerr << "Line " << Current_Line_Number_
			 << ": There must be at least one item." << endl;
		    Number_Of_Errors_++;
		    read(input);  // Follow the sub-itemization recursively
		}

	    } else if (input->eof() || (strlen(Buffer_) == 0)) {
	        cerr << "Line " << Current_Line_Number_
		     << ": There should be an </ITEMIZE> tag." << endl;
		Number_Of_Errors_++;
		return;

	    } else {
	        item = new ItemizeElement(Buffer_);
		elements_.push_back(item);
		number_of_items_++;
	    }

	    line_of_environment++;
	}

	input->getline(Buffer_,BUFFER_SIZE);
	Current_Line_Number_++;
    }

    if (line_of_environment == 1) {
        cerr << "Line " << Current_Line_Number_
	     << ": There must be at least one item." << endl;
	Number_Of_Errors_++;
    }
}


void ItemizeEnvironment::write(ofstream *output,int number_of_items)
{
    if (number_of_items == -1) {
        writeAllItems(output,ItemizeElement::NORMAL);

    } else {
	writeItemize(output,(number_of_items > (signed)number_of_items_ ?
			     number_of_items_ : number_of_items));
    }
}

void ItemizeEnvironment::writeAllItems(ofstream *output,int style)
{
    writeHeader(output);
    ItemizeElement *item;

    for (unsigned int i=0;i<elements_.size();i++) {
        item = (ItemizeElement*)elements_[i];
        item->write(output,style);
	if (item->sub_itemize_) {
	    item->sub_itemize_->writeAllItems(output,style);
	}
    }

    *output << "</UL>" << endl;
}

void ItemizeEnvironment::writeItemize(ofstream *output,int number_of_items)
{
    writeHeader(output);

    ItemizeEnvironment *sub_itemize;
    unsigned int i=0;
    while ((number_of_items > 0) && (i < elements_.size())) {
        elements_[i]->write(output,(number_of_items-- == 1 ?
				    ItemizeElement::NORMAL :
				    ItemizeElement::WEAK));

	sub_itemize = ((ItemizeElement*)elements_[i++])->sub_itemize_;
	if (sub_itemize && number_of_items > 0) {
	    if (sub_itemize->shouldBeWrittenOneByOne()) {
	        sub_itemize->writeItemize(output,number_of_items);
		number_of_items -= sub_itemize->getNumberOfElements();

	    } else {
	        sub_itemize->writeAllItems(output,(number_of_items-- == 1 ?
						   ItemizeElement::NORMAL :
						   ItemizeElement::WEAK));
	    }
	}
    }

    *output << "</UL>" << endl;
}

void ItemizeEnvironment::writeHeader(ofstream *output)
{
    char *left_margin_size;

    int margin = getMaxLength() / 18;
    switch (margin) {
      case 0:
	left_margin_size = "tiny";
	break;

      case 1:
	left_margin_size = "medium";
	break;

      default:
	left_margin_size = "large";
	break;
    }

    *output << "<UL";
    if (with_left_margin_ && (level_ == 0)) {
        *output << " class=\"" << left_margin_size << "\"";
    }
    if (level_ == 1) {
        *output << " type=\"circle\"";
    }

    *output << ">" << endl;
}


int ItemizeEnvironment::shouldBeWrittenOneByOne()
{
    return write_one_by_one_;
}


int ItemizeEnvironment::getNumberOfElements()
{
    return number_of_items_;
}


unsigned int ItemizeEnvironment::getMaxLength()
{
    unsigned int length;
    unsigned int max_length = 0;
    ItemizeElement *item;

    for (unsigned int i=0;i<elements_.size();i++) {
        item = (ItemizeElement*)elements_[i];
        length = item->getLength();
	if (length > max_length)  max_length = length;

	if (item->sub_itemize_) {
	    length = item->sub_itemize_->getMaxLength();
	    if (length > max_length)  max_length = length;
	}
    }

    return max_length;
}
