#include <config.h>

/*hpux*/
#ifndef _OFF_T
#  define _OFF_T
   typedef long off_t;
#endif /* _OFF_T */
/*hpux*/



#include <dirent.h>
#include <getopt.h>
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>

#include "main.h"
#include "parameters.h"
#include "presentation.h"

#include <magick/magick.h>


char Buffer_[BUFFER_SIZE];

unsigned int Current_Line_Number_;
unsigned int Number_Of_Errors_;

Parameters *Parameters_;
Presentation *Presentation_;


void print_help_text()
{
    cout << endl << "Usage: prestimel [options] inputfile" << endl

	 << "  -v, --version : Print version and exit." << endl

	 << "  -h, --help : Print help and exit." << endl

	 << "  -p path, --path-to-themes=path : Set the path to the themes." << endl
	 << "  -t[theme], --theme[=theme] : Use the theme \"theme\". If no theme is given," << endl
	 << "                               print a list of all available themes." << endl
	 << "  -r resolution, --resolution=resolution : Sets the resolution" << endl
	 << "                                           (800x600 or 1024x768)" << endl
	 << "  -B, --disable-background : Disable the background-graphic for the slides" << endl
	 << "  -H, --disable-header : Disable the background-graphic for the header" << endl
	 << "  -F, --disable-footer : Disable the background-graphic for the footer" << endl;
}


void list_available_themes()
{
    DIR *directory,*directory2 = NULL;
    dirent *entry,*entry2;
    ostrstream *dir_name;

    cout << endl;

    if (Parameters_->path_to_themes_ == NULL) {
        cout << "I cannot list the available themes, "
	     << "because the theme-path is uninitialized." << endl;
        return;
    }
    if ((directory = opendir(Parameters_->path_to_themes_)) == NULL) {
        cerr << "The theme-path \"" << Parameters_->path_to_themes_
	     << "\" doesn't exist." << endl;
        return;
    }

    cout << "Available themes in \"" << Parameters_->path_to_themes_
	 << "\" :" << endl;

    unsigned int number_of_themes = 0;
    while ((entry = readdir(directory)) != NULL) {
        if (directory2)  closedir(directory2);
	dir_name = new ostrstream;
	*dir_name << Parameters_->path_to_themes_ << '/' << entry->d_name
		  << ends;
	directory2 = opendir(dir_name->str());
	dir_name->freeze(0);
	delete dir_name;

	if (directory2) {
	    while ((entry2 = readdir(directory2)) != NULL) {
	        // Check, if the config-file exists
	        if (!strcmp("config",entry2->d_name)) {
		    cout << "    " << entry->d_name << endl;
		    number_of_themes++;
		}
	    }
	}
    }
    closedir(directory);

    if (number_of_themes > 0) {
        cout << endl << "Select a theme with the option \"-ttheme\" or "
	     << "\"--theme=theme\"." << endl;
    } else {
        cout << endl << "There are no themes available." << endl;
    }
}


int parse_commandline_options(int argc, char **argv)
{
    option long_options[] = {
        { OPTION_PATH_TO_THEMES,1,0,'p' },
        { OPTION_THEME,2,0,'t' },
        { OPTION_TARGET_DIRECTORY,1,0,'d' },
        { OPTION_RESOLUTION,1,0,'r' },
	{ OPTION_DISABLE_BACKGROUND,0,0,'B' },
	{ OPTION_DISABLE_HEADER,0,0,'H' },
	{ OPTION_DISABLE_FOOTER,0,0,'F' },
	{ OPTION_HELP,0,0,'h' },
	{ OPTION_VERSION,0,0,'v' },
        { 0,0,0,0 }
    };
    int option_index;
    int c;

    while ((c = getopt_long(argc,argv,"hvBHFp:t::d:r:",
			    long_options,&option_index)) != -1) {
        switch (c) {
	  case 'v':
	    cout << endl << "This is " << PACKAGE
		 << " version " << VERSION << endl;
	    return -1;
	    break;
	    
	  case 'B':
	    Parameters_->enable_background_graphic_ = 0;
	    break;

          case 'H':
	    Parameters_->enable_header_graphic_ = 0;
	    break;

	  case 'F':
	    Parameters_->enable_footer_graphic_ = 0;
	    break;
	  
          case 'p':
	    Parameters_->setPathToThemes(optarg);
	    break;

          case 't':
            if (optarg == NULL) {
	        list_available_themes();
		return -1;
	    } else {
	        Parameters_->setTheme(optarg);
	    }
	    break;

          case 'd':
	    Parameters_->setTargetDirectory(optarg);
	    break;

	  case 'r':
	    if (!strcmp(optarg,"800x600")) {
	        Parameters_->resolution_ = 800;
	    } else if (!strcmp(optarg,"1024x768")) {
	        Parameters_->resolution_ = 1024;
	    } else {
	        cerr << "The resolution must be 800x600 or 1024x768." << endl;
		return -1;
	    }
	    break;

          default:
	    print_help_text();
	    return -1;
	}
    }

    if (Parameters_->initPathToTheme()) {
	cerr << endl << "Use the option \"-t\" or \"--theme\" to see a list "
	     << "of all available themes." << endl;
	return -1;
    }

    if (optind < argc) {
        Parameters_->input_file_ = new char[strlen(argv[optind])+1];
	strcpy(Parameters_->input_file_,argv[optind]);
    } else {
        cerr << endl << "Error: No inputfile specified." << endl;
	return -1;
    }

    return 0;
}


int main(int argc, char **argv)
{
    SetErrorHandler(NULL);
    SetWarningHandler(NULL);


    // Initialize all global variables.
    Parameters_ = new Parameters();
    Presentation_ = new Presentation();
    Current_Line_Number_ = 0;
    Number_Of_Errors_ = 0;


    // Read the global config-file and update the parameters.
    Parameters_->readConfigFile(GLOBAL_CONFIG_FILE);

    // Read the user config-file, which is located in the home-directory.
    // The given options will override the options of the global config-file.
    ostrstream user_config_file;
    user_config_file << getenv("HOME") << '/' << USER_CONFIG_FILE << ends;
    Parameters_->readConfigFile(user_config_file.str());
    user_config_file.freeze(0);

    // Evaluate the commandline-options.
    // The given options will override the options of the config-files.
    if (parse_commandline_options(argc,argv)) {
	delete Parameters_;
	delete Presentation_;
	return -1;
    }


    // Do some final initializations.
    Parameters_->initImageSizes();
    cout << endl;


    // Now read the presentation...
    if (Presentation_->read() == 0) {
        // ...and write the HTML-files, if the inputfile is OK
        Presentation_->write();
    }


    // Clean up everything
    delete Parameters_;
    delete Presentation_;

    return 0;
}


int fcopy(const char *src,const char *dst)
{
    char copy_buffer[4096];
    size_t characters_read,characters_written;
    int return_value = 0;

    FILE *f_src = fopen(src,"r");
    if (!f_src) {
        cerr << "Error : Could not open the file \"" << src << "\"." << endl;
	return -1;
    }
    FILE *f_dst = fopen(dst,"w");
    if (!f_dst) {
        cerr << "Error : Could not create the file \"" << dst << "\"." << endl;
	return -1;
    }

    while (!feof(f_src)) {
        characters_read = fread(copy_buffer,sizeof(char),4096,f_src);
	if (characters_read > 0) {
	    characters_written = fwrite(copy_buffer,sizeof(char),
					characters_read,f_dst);
	    if (characters_written != characters_read) {
	        cerr << "Error while writing to \"" << dst << "\". Disk full ?"
		     << endl;
		return_value = -1;
		break;
	    }
	}
    }

    fclose(f_src);
    fclose(f_dst);

    return return_value;
}
