#ifndef MAIN_H
#define MAIN_H


class Parameters;
class Presentation;


// Defines of the valid options in the config-files.
// These are also the long options for the commandline-parameters.
#define OPTION_PATH_TO_THEMES "path-to-themes"
#define OPTION_THEME "theme"
#define OPTION_TARGET_DIRECTORY "target-directory"
#define OPTION_RESOLUTION "resolution"
// These options are only available as commandline-parameters.
#define OPTION_DISABLE_BACKGROUND "disable-background"
#define OPTION_DISABLE_HEADER "disable-header"
#define OPTION_DISABLE_FOOTER "disable-footer"
#define OPTION_HELP "help"
#define OPTION_VERSION "version"

// The filenames for the navigation-buttons
#define PREV "prev.gif"
#define NEXT "next.gif"

// The filenames of the theme-stylesheets
#define STYLES "styles"
#define STYLES_EXT ".css"

// The filenames of the background-images of the themes
#define BG_SLIDE "bg_slide"
#define BG_HEADER "bg_header"
#define BG_FOOTER "bg_footer"
#define GIF_EXT ".gif"
#define JPG_EXT ".jpg"


#define BUFFER_SIZE 200
/// A buffer which will be used for the reading of the inputfile
extern char Buffer_[BUFFER_SIZE];

/// Count the lines of the inputfile, to produce better error-messages
extern unsigned int Current_Line_Number_;

/// The number of errors in the inputfile
extern unsigned int Number_Of_Errors_;


/// This class holds all parameters
extern Parameters *Parameters_;

/// A global pointer to the presentation
extern Presentation *Presentation_;



/** This function lists a short help for all commandline-options.
*/
extern void print_help_text();

/** This function lists all available themes in the current theme-directory.
*/
extern void list_available_themes();

/** This function parses the given commandline-parameters and updates the
    variables in the class Parameters_.
    @param argc The number of parameters of the commandline
    @param argv The parameter-array
    @return 0, if the parameters are correct, else -1
*/
extern int parse_commandline_options(int argc, char **argv);

/** This is the main function.
    @param argc The number of parameters of the commandline
    @param argv The parameter-array
    @return 0, if the program runs through, else -1
*/
extern int main(int argc,char **argv);

/** This function copies a file.
    @param src The filename of the source-file
    @param dst The filename of the destination-file
    @return 0, if everything is OK, else -1.
*/
extern int fcopy(const char *src, const char *dst);


#endif //MAIN_H
