#ifndef PRESENTATION_H
#define PRESENTATION_H


#include "slide.h"


/** This class represents a whole presentation.
    It contains a theme and a vector which holds pointers to all slides.
 */
class Presentation {
  public:
    /// Default constructor
    Presentation();

    /// Destructor
    ~Presentation();


    /** This method reads the presentation from the inputfile.
	The filename is given by the member variable input_file_ of the class
	Parameters.
	@return -1, if the file doesn't exist, else the number of errors in
	        the inputfile.
    */
    int read();

    /** This method writes the whole presentation.
	The theme-files and the HTML-files for each slide will be copied
	to the target directory, which is given by the member variable
	target_directory_ of the class Parameters.
    */
    void write();

    /** This method returns the previous slide of the slide with the given
	number.
	@param slide_number The number of the slide, which next one should be
	       returned
	@return The previous slide, or NULL if the given slide is the first one
    */
    Slide *getPrevSlide(unsigned int slide_number);

    /** This method returns the next slide of the slide with the given number.
	@param slide_number The number of the slide, which previous one should
	       be returned
	@return The next slide, or NULL if the given slide is the last one
    */
    Slide *getNextSlide(unsigned int slide_number);

  private:
    /** This helper-method will write an index-page.
	It is called in the method write() before writing the slides.
    */
    void writeIndexPage();


  private:
    /// A vector which contains pointers to all slides
    vector<Slide*> slides_;
};

#endif //PRESENTATION_H
