#ifndef SLIDE_H
#define SLIDE_H


#include "environment.h"


/** This class represents one slide.
    It contains all data, that is needed for writing the apropriate HTML-File.
    A Slide can also consist of subsildes, which will be written to own
    HTML-files.
    Note : The subslides itself are generated on-the-fly, when the method
    write() is called. They are not stored in a list, or something like that.
*/
class Slide {
  public:
    /** Creates a slide with the given sequence-number.
	@param slide_number The sequence-number of this slide
    */
    Slide(unsigned int sequence_number);

    /// Destructor
    ~Slide();


    /** This method creates a slide out of the given inputfile.
	The inputfile will be read until an empty line, which indicates the
	beginning of the next slide.
	@param input The inputfile, which contains all slides
    */
    void read(ifstream *input);

    /** This method writes the slide to a HTML-file. If the slide contains
	subslides, all subslides will be written to own HTML-files.
	The writing itself is done by calling the method writeSubSlide().
    */
    void write();

    /** This method returns the title of the slide.
	@return The title of the slide
    */
    char *getTitle();

  private:
    /** This helper-method writes a (sub)slide to a HTML-File.
	The filename of the HTML-file is generated on-the-fly by the value
	of sequence_number_ (and the value of the parameter sub_slide).
	@param sub_slide The subslide that should be written. If -1, there
	       doesn't exist any subslides and the whole slide will be printed.
    */
    void writeSubSlide(int sub_slide);

    /** This helper-method prints the HTML-header of the (sub)slide to the
	given output filestream.
	@param output The output filestream.
	@param prev_slide The filename of the previous (sub)slide
	@param next_slide The filename of the next (sub)slide
    */
    void writeHTMLHeader(ofstream *output,char *prev_slide,char *next_slide);

    /** This helper-method prints the header of the (sub)slide to the
	given output filestream.
	@param output The output filestream
    */
    void writeHeader(ofstream *output);

    /** This helper-method prints the footer of the (sub)slide to the
	given output filestream.
	@param output The output filestream
	@param prev_slide The filename of the previous (sub)slide
	@param next_slide The filename of the next (sub)slide
    */
    void writeFooter(ofstream *output,char *prev_slide,char *next_slide);


  public:
    /** The sequence-number of this slide.
	The first slide will have the sequence-number 1.
    */
    unsigned int sequence_number_;

    /// The number of subslides. If there are no subslides, the value is -1.
    int number_of_subslides_;

  private:
    /// The title of the slide
    ostrstream title_;

    /// A pointer to any environment
    Environment *environment_;
};

#endif //SLIDE_H
